# Documentation string. I wonder what ever refers to it?

testString = """
http://www.namshub.org:8080/test/manila/message/create?username=""&password=""&sitename=""&subject="test"&body="test%20test%20test"&bodyType="text/plain"&inResponseTo=""&windowInfo=""&rendererInfo=""
"""

__doc__ = """ZopeFish product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.

See README.txt for more details."""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

from OFS.DTMLMethod import DTMLMethod

#from ZODB.Transaction import Transaction, get_transaction

import OFS.SimpleItem
import OFS.Folder
import Acquisition
import AccessControl.Role

import re
import string
from xmlrpclib import True, False, Server

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

manage_addZopeFishForm = HTMLFile('zopefishAdd', globals()) 

def manage_addZopeFish(self, id, title='', REQUEST=None):
    """Add a ZopeFish to a folder."""
    if title == '':
        title = 'ZopeFish!'
    self._setObject(id, ZopeFish(id, title, REQUEST))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

"""Step #2: Provide the class for your ZopeFish object."""

class DocumentControl(
    OFS.Folder.Folder,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager
    ):
    """documentcontrol"""

    default = ""
    current = ""
    ownerName = ""
    ownerPassword = ""
    owned = 0

    meta_type = "Pike Compatible XML-RPC Document Control Responder"

    manage_main = HTMLFile('leafnodesDisplay', globals()) # Management Interface

    def __init__(self, defaultString, container, id, parent):
        """__init__"""
        self.default = defaultString
        self.current = defaultString
        self.container = container
        self.ownerName = ""
        self.ownerPassword = ""
        self.owned = 0
        self.parent = parent
        self.id = id
        self.title = id

    def manage_afterAdd(self, item, container): pass
    def manage_afterClone(self, item): pass
    def manage_beforeDelete(self, item, container): pass

    def objectItems(self, type=None):
        """   """
        return [("checkIn", self.checkIn),
                ("get", self.get),
                ("restoreDefault", self.restoreDefault),
                ("set", self.set)]

    def objectValues(self, type=None):
        """   """
        return [self.checkIn, self.get, self.restoreDefault, self.set]

    def objectIds(self, type=None):
        """   """
        return [self.checkIn.id, self.get.id, self.restoreDefault.id, self.set.id]

    def checkIn(self, username, password, siteName):
        """checkIn"""
        print "manila." + self.id + ".checkIn:"
        print "\tusername    :", username
        print "\tpassword    :", password
        print "\tsitename    :", siteName
        print "---------------------"
        return True

    def get(self, username, password, siteName):
        """get"""
        print "manila." + self.id + ".get:"
        print "\tusername    :", username
        print "\tpassword    :", password
        print "\tsitename    :", siteName
        print "---------------------"
        return self.current

    def restoreDefault(self, username, password, sitename):
        """restoreDefault"""
        print "manila." + self.id + ".restoreDefault:"
        print "\tusername    :", username
        print "\tpassword    :", password
        print "\tsitename    :", sitename
        print "---------------------"
        self.current = self.default

    def set(self, username, password, sitename, s, windowInfo):
        """set"""
        print "manila." + self.id + ".set:"
        print "\tusername    :", username
        print "\tpassword    :", password
        print "\tsitename    :", sitename
        print "\ts           :", s
        print "\twindowInfo  :", windowInfo
        print "---------------------"
        self.current = s

class TemplateControl(DocumentControl):
    """   """
    
    def restoreDefault(self, username, password, sitename):
        """restoreDefault"""
        DocumentControl.restoreDefault(self, username, password, sitename)
        self.parent.template.manage_edit(self.current, "template")
        return True
        
    def set(self, username, password, sitename, s, windowInfo):
        """set"""
        DocumentControl.set(self, username, password, sitename, s, windowInfo)
        self.parent.template.manage_edit(self.current, "template")
        return True

class Messages(
    OFS.Folder.Folder,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager
    ):
    """messages"""

    meta_type = "Pike Compatible XML-RPC Message Responder"

    manage_main = HTMLFile('leafnodesDisplay', globals()) # Management Interface
    manage_workspace = manage_main

    def __init__(self, id, parent):
        """   """
        self.parent = parent
        self.id = id
        self.title = id

    def manage_afterAdd(self, item, container): pass
    def manage_afterClone(self, item): pass
    def manage_beforeDelete(self, item, container): pass

    def objectItems(self, type=None):
        """   """
        return [("addToStoriesList", self.addToStoriesList),
                ("checkIn", self.checkIn),
                ("isStory", self.isStory),
                ("removeFromStoriesList", self.removeFromStoriesList),
                ("create", self.create),
                ("set", self.set)]

    def objectValues(self, type=None):
        """   """
        return [self.addToStoriesList,
                self.checkIn,
                self.isStory,
                self.removeFromStoriesList,
                self.create,
                self.set]

    def objectIds(self, type=None):
        """   """
        return [self.addToStoriesList.id,
                self.checkIn.id,
                self.isStory.id,
                self.removeFromStoriesList.id,
                self.create.id,
                self.set.id]

    def addToStoriesList(self, username, password, sitename, msgNum):
        """addToStoriesList"""
        print "manila." + self.id + ".addToStoriesList:"
        print "\tusername:", username
        print "\tpassword:", password
        print "\tsitename:", sitename
        print "\tmsgNum  :", msgNum
        print "-----------------"
        return True

    def checkIn(self, username, password, siteName, msgNum):
        """checkIn"""
        print "manila." + self.id + ".checkIn:"
        print "\tusername:", username
        print "\tpassword:", password
        print "\tsiteName:", siteName
        print "\tmsgNum  :", msgNum
        print "-----------------"
        return True

    def isStory(self, username, password, sitename, msgNum):
        """isStory"""
        print "manila." + self.id + ".isStory:"
        print "\tusername:", username
        print "\tpassword:", password
        print "\tsitename:", sitename
        print "\tmsgNum  :", msgNum
        print "-----------------"
        return False

    def removeFromStoriesList(self, username, password, siteName, msgNum):
        """removeFromStoriesList"""
        print "manila." + self.id + ".removeFromStoriesList:"
        print "\tusername:", username
        print "\tpassword:", password
        print "\tsitename:", siteName
        print "\tmsgNum  :", msgNum
        print "-----------------"
        return True

    def create(self, username, password, sitename, subject, body,
               bodyType, inResponseTo, windowInfo, rendererInfo):
        """create"""
        print "manila." + self.id + ".create:"
        print "\tusername    :", username
        print "\tpassword    :", password
        print "\tsitename    :", sitename
        print "\tsubject     :", subject
        print "\tbody        :", body
        print "\tbodyType    :", bodyType
        print "\tinResponseTo:", inResponseTo
        print "\twindowInfo  :", windowInfo
        print "\trendererInfo:", rendererInfo
        print "---------------------"
        headers = self.makeHeaders(subject, bodyType, inResponseTo, rendererInfo, windowInfo)
        msgnum = self.parent.discuss.addNextMessage(body, headers)
        storyInfo = self.parent.getStoryInfo(None, subject, body, msgnum, username)
        return storyInfo

    def set(self, username, password, sitename, msgnum, subject, body,
            bodyType, windowInfo, rendererInfo):
        """set"""
        print "manila." + self.id + ".set:"
        print "\tusername    :", username
        print "\tpassword    :", password
        print "\tsitename    :", sitename
        print "\tmsgnum      :", msgnum
        print "\tsubject     :", subject
        print "\tbody        :", body
        print "\tbodyType    :", bodyType
        print "\twindowInfo  :", windowInfo
        print "\trendererInfo:", rendererInfo
        print "---------------------"
        msg = self.parent.discuss.getMessage(msgnum)
        msg.setTitle(subject)
        headers = msg.headers
        self.parent.discuss.addMessage(msgnum, body, headers)
        storyInfo = self.parent.getStoryInfo(None, subject, body, msgnum, username)
        return storyInfo

    def makeHeaders(self, subject, bodyType, inResponseTo, rendererInfo, windowInfo):
        """makeHeaders"""
        headers = {}
        headers['subject'] = subject
        headers['bodyType'] = bodyType
        headers['inResponseTo'] = inResponseTo
        headers['rendererInfo'] = rendererInfo
        headers['windowInfo'] = windowInfo
        return headers

class PikeCompatibility(
    OFS.Folder.Folder,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager
    ):
    """The Pike Compatible XML-RPC interface"""

    meta_type = "Pike Compatible XML-RPC Interface"

    manage_main = HTMLFile('zopefishEdit', globals()) # Management Interface
    manage_workspace = manage_main

    def __init__(self, id, parent):
        self.parent = parent
        self.id = id
        self.title = "Pike Compatible XML-RPC Interface"
        self.css = DocumentControl(
            defaultCss,
            None,
            "css",
            self.parent)
        self.customPrefs = DocumentControl(
            defaultCustomPrefs,
            None,
            "customPrefs",
            self.parent)
        self.javaScript = DocumentControl(
            defaultJavaScript,
            None,
            "javaScript",
            self.parent)
        self.joinNowMembersBox = DocumentControl(
            defaultJoinNowMembersBox,
            None,
            "joinNowMembersBox",
            self.parent)
        self.message = Messages("message", self.parent)
        self.navigation = DocumentControl(
            defaultNavigation,
            None,
            "navigation",
            self.parent)
        self.newsDayTemplate = DocumentControl(
            defaultNewsDayTemplate,
            None,
            "newsDayTemplate",
            self.parent)
        self.signOutMembersBox = DocumentControl(
            defaultSignOutMembersBox,
            None,
            "signOutMembersBox",
            self.parent)
        self.siteStructure = DocumentControl(
            defaultSiteStructure,
            None,
            "siteStructure",
            self.parent)
        self.template = TemplateControl(
            defaultTemplate,
            self.parent.template,
            "template",
            self.parent)

    def manage_afterAdd(self, item, container): pass
    def manage_afterClone(self, item): pass
    def manage_beforeDelete(self, item, container): pass

    def objectItems(self, type=None):
        """   """
        return [(self.css.id, self.css),
                (self.customPrefs.id, self.customPrefs),
                (self.javaScript.id, self.javaScript),
                (self.joinNowMembersBox.id, self.joinNowMembersBox),
                (self.message.id, self.message),
                (self.navigation.id, self.navigation),
                (self.newsDayTemplate.id, self.newsDayTemplate),
                (self.signOutMembersBox.id, self.signOutMembersBox),
                (self.siteStructure.id, self.siteStructure),
                (self.template.id, self.template)]

    def objectValues(self, type=None):
        """   """
        return [self.css,
                self.customPrefs,
                self.javaScript,
                self.joinNowMembersBox,
                self.message,
                self.navigation,
                self.newsDayTemplate,
                self.signOutMembersBox,
                self.siteStructure,
                self.template]

    def objectIds(self, type=None):
        """   """
        return [self.css.id,
                self.customPrefs.id,
                self.javaScript.id,
                self.joinNowMembersBox.id,
                self.message.id,
                self.navigation.id,
                self.newsDayTemplate.id,
                self.signOutMembersBox.id,
                self.siteStructure.id,
                self.template.id]

class ZopeFishMessage(
    OFS.SimpleItem.Item,
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ):
    """ZopeFishMessage"""

    meta_type = "ZopeFish Message"
    msg = ""
    headers = {}

    body = HTMLFile('zopefishMessage', globals())

    def __init__(self, theMsg, theHeaders, key, parent):
        """__init__"""
        self.msg = theMsg
        self.headers = theHeaders
        if self.headers.has_key('subject'):
            self.title = self.headers['subject']
        else:
            self.title = key
        self.id = key
        self.key = key
        self.parent = parent
        self.index_html = self.parent.template

    def setTitle(self, title):
        """   """
        self.title = title
        self.headers['subject'] = title

    def readMessageTag(self, tag=None):
        """readMessageTag"""
        if tag is None:
            tag = self.id
        return "<a href=\"" + \
               self.parent.location + \
               "/discuss/" + \
               self.id + \
               "\">" + \
               tag + \
               "</a>"

    def editInTextArea(self, REQUEST=None):
        """   """
        pass

    def editInTextAreaTag(self, tag):
        """editInTextAreaTag"""
        return "<a href=\"" + \
               self.parent.location + \
               "/discuss/" + \
               self.id + \
               "/editInTextArea\">" + \
               tag + \
               "</a>"

    def editInPike(self, REQUEST=None):
        """   """
        storyInfo = self.parent.getStoryInfo(REQUEST, self.title, self.msg, self.id)
        siteInfo = self.parent.getSiteInfo(REQUEST)
        self.parent.getPikeServer(REQUEST).pike.editStory(siteInfo, storyInfo)
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect(self.parent.location + "/discuss/" + self.id)
        else:
            return self.index_html(REQUEST)

    def editInPikeTag(self, tag):
        """editInPikeTag"""
        return "<a href=\"" + \
               self.parent.location + \
               "/discuss/" + \
               self.id + \
               "/editInPike\">" + \
               tag + \
               "</a>"

class MessageCollection(
    OFS.Folder.Folder,
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ):
    """discussion group"""

    meta_type = "ZopeFish Message Collection"
    currentNum = 0

    manage_main = HTMLFile("discussEdit", globals())
    manage_workspace = manage_main

    def __init__(self, id, parent):
        """__init__"""
        self.parent = parent
        self.currentNum = 0
        self.id = id
        self.title = "Message Collection"
        self.index_html = self.parent.template

    def _getNextMessageKey(self):
        nextNum = self.currentNum
        self.currentNum = self.currentNum + 1
        return "msg%06d" % nextNum

    def addNextMessage(self, msg, headers):
        """   """
        msgnum = self._getNextMessageKey()
        self.addMessage(
            msgnum,
            msg,
            headers)
        return msgnum

    def addNextMessageNoHeaders(self, msg):
        """   """
        return self.addNextMessage(msg, {})

    def addMessage(self, key, msg, headers):
        """   """
        msgs = self.__dict__
        msgs[key] = ZopeFishMessage(msg, headers, key, self.parent)
        self.__dict__ = msgs
        self.parent.resync()

    def addMessageNoHeader(self, key, msg):
        """   """
        self.addMessage(key, msg, {})

    def checkMessage(self, key):
        """   """
        if not self.__dict__.has_key(key):
            raise AttributeError

    body = HTMLFile("discussIndex", globals())

    def newMessageInPikeTag(self, tag):
        """newMessageTag"""
        return "<a href=\"" + \
               self.parent.location + \
               "/discuss/newMessageInPike\">" + \
               tag + \
               "</a>"

    def newMessageInPike(self, REQUEST=None):
        """   """
        storyInfo = self.parent.getStoryInfo(REQUEST, "New Pike Message", "", "msg000000")
        siteInfo = self.parent.getSiteInfo(REQUEST)
        self.parent.getPikeServer(REQUEST).pike.newStory(siteInfo, storyInfo)

    def getMessage(self, msgnum, REQUEST=None):
        """   """
        self.checkMessage(msgnum)
        msg = self.__dict__[msgnum]
        if REQUEST is not None:
            return msg.index_html(REQUEST)
        else:
            return msg

    def manage_afterAdd(self, item, container): pass
    def manage_afterClone(self, item): pass
    def manage_beforeDelete(self, item, container): pass

    def objectItems(self, type=None):
        """objectItems"""
        list = []
        for k in self.__dict__.keys():
            if k[0:3] == "msg":
                list.append((k, self.__dict__[k]))
        return list

    def objectValues(self, type=None):
        """objectValues"""
        list = []
        for k in self.__dict__.keys():
            if k[0:3] == "msg":
                list.append(self.__dict__[k])
        return list

    def objectIds(self, type=None):
        """objectValues"""
        list = []
        for k in self.__dict__.keys():
            if k[0:3] == "msg":
                list.append(k)
        return list

class StoryReader:
    """StoryReader"""
    pass

class ZopeFish(
    OFS.Folder.Folder,
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ): 
    """ZopeFish object. 
    
    This is an attempt at writing a product from scratch using mostly 
    Python instead of ZClasses. We'll see how it all goes.
    """
        
    meta_type = 'ZopeFish' # what do people think they're adding? 

    manage_options = ( # what management options are there?
	{'label': 'Edit',       'action': 'manage_main'},
	{'label': 'View',       'action': ''}, # defaults to index_html
	{'label': 'Security',   'action': 'manage_access'},
        ) 

    __ac_permissions__=( # what permissions make sense for us? 
	('View management screens', ('manage_tabs','manage_main')),
	('Change permissions',      ('manage_access',)           ),
	('Change ZopeFishes',       ('manage_edit',)             ),
	('View ZopeFishes',         ('',)                        ),
	)

    def __init__(self, id, title='', REQUEST=None): 
        """initialise a new instance of ZopeFish"""
        self.parent = self
        self.id = id
        self.title = title
        self.template = DTMLMethod()
        self.template.manage_edit(defaultTemplate, "template")
        self.index_html = self.template

        if REQUEST is not None:
            self.location = REQUEST.URL3 + "/" + self.id
            base = self.location[7:]
            index = string.find(base, "/")
            self.locationNoHost = base[index:]
        else:
            self.location = "unknown"

        self.discuss = MessageCollection("discuss", self)
        self.manila = PikeCompatibility("manila", self)

    def resync(self):
        """resync"""
        discuss = self.discuss
        self.discuss = discuss
        manila = self.manila
        self.manila = manila

    def allMessages(self):
        """allMessages"""
        return self.discuss.index_html()

    def manage_afterAdd(self, item, container): pass
    def manage_afterClone(self, item): pass
    def manage_beforeDelete(self, item, container): pass

    def objectItems(self, type=None):
        """   """
        return [(self.discuss.id, self.discuss), (self.manila.id, self.manila)]

    def objectValues(self, type=None):
        """   """
        return [self.discuss, self.manila]

    def objectIds(self, type=None):
        """   """
        return [self.discuss.id, self.manila.id]

    def getMember(self, REQUEST):
        """   """
        if REQUEST is not None:
            return REQUEST.AUTHENTICATED_USER.getUserName()
        else:
            return "test member"

    def getStoryInfo(self, REQUEST, subject, body, msgNum, member=None):
        """   """
        if member is None:
            thisMember = self.getMember(REQUEST)
        else:
            thisMember = member

        if REQUEST is None:
            url = self.location
        else:
            url = REQUEST.URL2

        # this is a hack.
        if string.find(url, "/discuss") == -1:
            url = url + "/discuss"

        storyInfo = {}
        storyInfo['body'] = body
        storyInfo['member'] = thisMember
        storyInfo['msgnum'] = msgNum
        storyInfo['postTime'] = ""
        storyInfo['lastUpdate'] = ""
        storyInfo['ctReads'] = 0
        storyInfo['subject'] = subject
        storyInfo['responses'] = []
        storyInfo['URL'] = url + "/" + msgNum
        return storyInfo

    def getSiteInfo(self, REQUEST):
        """   """
        host = REQUEST.HTTP_HOST
        i = string.find(host, ":")
        if i != -1:
            host = host[0:i]

        url = REQUEST.URL2
        if url[-1] != "/":
            url = url + "/"

        siteInfo = {}
        siteInfo['host'] = host
        siteInfo['port'] = REQUEST.SERVER_PORT
        siteInfo['rpcPath'] = self.locationNoHost
        siteInfo['siteName'] = self.zopeFishId()
        siteInfo['displaySiteName'] = self.zopeFishTitle()
        siteInfo['username'] = self.getMember(REQUEST)
        siteInfo['password'] = ""
        siteInfo['URL'] = url
        return siteInfo

    def getPikeServer(self, REQUEST=None):
        """   """
        if REQUEST is not None:
            #print "addr:", REQUEST.REMOTE_ADDR
            return Server("http://" + REQUEST.REMOTE_ADDR + ":81")
        else:
            return Server(myPike)

    def zopeFishId(self):
        """zopeFishId"""
        return self.parent.id

    def zopeFishTitle(self):
        """zopeFishTitle"""
        return self.parent.title

    def editTemplateInPikeTag(self, tag=None):
        """editTemplateTag"""
        if tag is None:
            tag = "Edit Template"
        return "<a href=\"" + \
               self.location + \
               "/editTemplateInPike\">" + \
               tag + \
               "</a>"

    def editTemplateInPike(self, REQUEST=None):
        """editTemplateInPike"""
        if REQUEST is None:
            return
        body = self.template.read()
        body = re.sub("\r\n", "\n", body)
        body = re.sub("\n", "\r", body)
        itemInfo = {}
        itemInfo['body'] = body
        itemInfo['type'] = 'Template'
        #itemInfo['outline'] = self.template.read()
        siteInfo = self.getSiteInfo(REQUEST)
        pikeServer = self.getPikeServer(REQUEST)
        pikeServer.pike.editAdvancedItem(siteInfo, itemInfo)

    body = HTMLFile('index', globals()) # View Interface
    #template = HTMLFile("template", globals())
    
    manage_main = HTMLFile('zopefishEdit', globals()) # Management Interface
    manage_workspace = manage_main
    
    def manage_edit(self, title, REQUEST=None): 
        """does this really need a doc string?"""
        self.title = title
	if REQUEST is not None:
	    return MessageDialog(
		title = 'Edited',
		message = "Properties for %s changed." % self.id,
		action = './manage_main',
		)

defaultCss = """
"""

defaultCustomPrefs = """
"""

defaultJavaScript = """
"""

defaultJoinNowMembersBox = """
"""

defaultNavigation = """
"""

defaultNewsDayTemplate = """
"""

defaultSignOutMembersBox = """
"""

defaultSiteStructure = """
"""

defaultTemplate = """
<dtml-var standard_html_header>

<table width="100%" border="0" cellpadding="5" cellspacing="0">
	<tr bgcolor="darkgray">
		<td>
			<dtml-var zopeFishTitle>
			</td>
		<td align="right">
			<dtml-var zopeFishId>
                        </td>
		</tr>
	</table>

<table width="100%" border="0" cellpadding="5" cellspacing="0">
	<tr>
		<td width="35">
			&nbsp;<br>
			</td>
		<td>
			&nbsp;
			<br>
			<dtml-var body>
                        </td>
		</tr>
	</table>

<p>
<dtml-var editTemplateInPikeTag>

<dtml-var standard_html_footer>"""

